- test:
    timeout:
      minutes: {{ timeout }}
    interactive:
    - name: virtiop9
      prompts: ["{{ '", "'.join(prompts)}}"]
      script:
      - command: "mkdir dummy && mount -t 9p -o trans=virtio,version=9p2000.L FM dummy && cat dummy/environment"
        name: virtiop9
        successes:
        - message: "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games:/usr/local/games:/snap/bin"
