- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository: {{ test_definitions }}
      from: url
      compression: zstd
      path: automated/linux/gpiod/gpiod.yaml
      name: libgpiod

