- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository: {{ test_definitions }}
      from: url
      compression: zstd
      path: automated/linux/kselftest/kselftest.yaml
      parameters:
        TST_CMDFILES: "{{ cmdfile }}"
        KSELFTEST_PATH: "/opt/kselftests/default-in-kernel"
        SKIPFILE: "skipfile-lkft.yaml"
        ENVIRONMENT: "production"
      name: {{ name }}

