- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository: {{ test_definitions }}
      from: url
      compression: zstd
      path: automated/linux/ltp/ltp.yaml
      parameters:
        SKIP_INSTALL: 'true'
        TST_CMDFILES: '{{ cmdfile|default(name) }}'
        SKIPFILE: '{{ parameters.get("SKIPFILE", "/tuxtest/skipfiles/" + arch + "/" + name) }}'
        ENVIRONMENT: 'production'
        TIMEOUT_MULTIPLIER: '3'
        LTP_TMPDIR: '/scratch'
        LTP_INSTALL_PATH: '/opt/ltp/'
      name: {{ name }}

